<?php
// +----------------------------------------------------------------------
// | 微服务-配置调用类
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

class Rpc
{
    private static $instance = null; // 创建静态对象变量,用于存储唯一的对象实例  
    private function __construct(){} // 私有化构造函数，防止外部调用
    private function __clone(){}     // 私有化克隆函数，防止外部克隆对象
    private static $config; // 配置

    /**
     * 实例化对象方法，供外部获得唯一的对象
    */
    public static function run(){
        if (empty(self::$instance)) {
            require_once 'lifecycle/rpc_error.php';
            require_once 'RedisCache.php';
            self::$config = require 'config/rpc.php';
            self::$instance = new Rpc();
        }
        return self::$instance;
    }

    /**
     * 当检测失败时，回调的处理函数
     * @todo 无
     * @author 小黄牛
     * @version v1.2.24 + 2021.1.9
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    public static function ping_error($class, $function, $config, $status) {
        rpc_error($class, $function, $config, $status);
        return false;
    }

    /**
     * 读取配置
     * @todo 无
     * @author 小黄牛
     * @version v1.2.24 + 2021.1.9
     * @deprecated 暂不启用
     * @global 无
     * @param string $key 配置KEY
     * @return void
    */
    public static function get($key=null) {
        $redis_key = self::$config['redis_key'];
        $config = RedisCache::run()->get($redis_key);
        $config = $config ? json_decode($config, true) : array();

        if ($key) {
            if (isset($config[$key])) {
                return $config[$key];
            }
            return false;
        }
        return $config;
    }

    /**
     * 删除某个配置
     * @todo 无
     * @author 小黄牛
     * @version v1.2.24 + 2021.1.9
     * @deprecated 暂不启用
     * @global 无
     * @param string $key 配置KEY
     * @return void
    */
    public static function delete($key) {
        $redis_key = self::$config['redis_key'];
        $config = RedisCache::run()->get($redis_key);
        $config = $config ? json_decode($config, true) : array();

        if (isset($config[$key])) {
            unset($config[$key]);
            RedisCache::run()->set($redis_key, self::json_encode($config));
            return true;
        }
        
        return false;
    }

    /**
     * 更新某条配置
     * @todo 无
     * @author 小黄牛
     * @version v1.2.24 + 2021.1.9
     * @deprecated 暂不启用
     * @global 无
     * @param string $class 配置KEY
     * @param string $function 配置KEY
     * @param mixed $val 配置参数
     * @return void
    */
    public static function setOne($class, $function, $val) {
        $redis_key = self::$config['redis_key'];
        $config = RedisCache::run()->get($redis_key);
        $config = $config ? json_decode($config, true) : array();

        if (isset($config[$class][$function])) {
            $list = $config[$class][$function];
            foreach ($list as $k=>$v) {
                if ($v['title'] == $val['title']) {
                    $config[$class][$function][$k] = $val;

                    RedisCache::run()->set($redis_key, self::json_encode($config));
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 删除某条配置
     * @todo 无
     * @author 小黄牛
     * @version v1.2.24 + 2021.1.9
     * @deprecated 暂不启用
     * @global 无
     * @param string $class 配置KEY
     * @param string $function 配置KEY
     * @param mixed $val 配置参数
     * @return void
    */
    public static function deleteOne($class, $function, $val) {
        $redis_key = self::$config['redis_key'];
        $config = RedisCache::run()->get($redis_key);
        $config = $config ? json_decode($config, true) : array();

        if (isset($config[$class][$function])) {
            $list = $config[$class][$function];
            foreach ($list as $k=>$v) {
                if ($v['title'] == $val['title']) {
                    unset($config[$class][$function][$k]);

                    RedisCache::run()->set($redis_key, self::json_encode($config));
                    return true;
                }
            }
        }
        return false;
    }
    
	/**
	 * 转义JSON
	 * @todo 无
	 * @author 小黄牛
	 * @version v1.2.24 + 2021.1.9
	 * @deprecated 暂不启用
	 * @global 无
	 * @param string $
	 * @param mixed $
	 * @return void
	*/
	public static function json_encode($array) {
        if(version_compare(PHP_VERSION,'5.4.0','<')) {
            $str = json_encode($array);
            $str = preg_replace_callback("#\\\u([0-9a-f]{4})#i", function($matchs){
                return iconv('UCS-2BE', 'UTF-8', pack('H4', $matchs[1]));
            },$str);
            return $str;
        }else{
            return json_encode($array, JSON_UNESCAPED_UNICODE);
        }
    }
}